/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LEDPARAMCONTROLLER_H
#define LEDPARAMCONTROLLER_H

#include <Controller.h>
#include "drivers/stereo_elec/StereoElecDriver.h"
#include "DAQParamController.h"

namespace stereo_daq {


/*!
 * \class LEDParamController
 */

class LEDParamController: public ExperimentController,
		public controller::Stoppable,
		public Select,
		public Set{

public:

	//! Type of controller
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 *
	 * It is the default constructor called by the creator class ExperimentControllerCreator.
	 *
	 * \param[in] name the name of the controller
	 */
	LEDParamController(const std::string& name);

	/*!
	 * \brief Copy constructor
	 *
	 * \param[in] controller the controller to copy
	 */
	LEDParamController(const LEDParamController& controller);

	/*!
	 * \brief Destructor
	 */
	virtual ~LEDParamController();

	/*!
	 * \brief Method called before changing the property value
	 *
	 * This method is called after setting configuration during the creation of controller.
	 */
	virtual void postConfiguration();

	/*!
	 * Properties
	 */
	Property<int32> delay;
	Property<int32> period;

	Property<bool> box1;
	Property<bool> box2;
	Property<bool> box3;
	Property<bool> box4;
	Property<bool> box5;
	Property<int32> pattern;

	Property<int32> nled;
	// 1 value for each fe board
	DynamicProperty<int32> dacbox1;
	DynamicProperty<int32> dacbox2;
	DynamicProperty<int32> dacbox3;
	DynamicProperty<int32> dacbox4;
	DynamicProperty<int32> dacbox5;

protected:


	DriverPtr<stereo_elec::StereoElecDriver> m_ELECDriver; //! The driver linked to the controller

	/*!
	* \brief select command
	*/
	virtual void select();

	/*!
	* \brief select command
	*/
	virtual void set();

	/*!
		 * \brief start
		 *
		 */
		virtual void start();

		/*!
		 * \brief Stop command
		 */
		virtual void stop();

private:

	/*
	 * local values
	 */
	bool m_StartActivated; //! Started activated flag

	void refreshDelayProperty(int32 value) throw (CannotSetValue);
	void refreshPeriodProperty(int32 value) throw (CannotSetValue);
	void refreshBox1Property(bool value) throw (CannotSetValue);
	void refreshBox2Property(bool value) throw (CannotSetValue);
	void refreshBox3Property(bool value) throw (CannotSetValue);
	void refreshBox4Property(bool value) throw (CannotSetValue);
	void refreshBox5Property(bool value) throw (CannotSetValue);
	void refreshPatternProperty(int32 value) throw (CannotSetValue);
	void refreshDACBox1Property(int32 index, int32 value) throw (CannotSetValue);
	void refreshDACBox2Property(int32 index, int32 value) throw (CannotSetValue);
	void refreshDACBox3Property(int32 index, int32 value) throw (CannotSetValue);
	void refreshDACBox4Property(int32 index, int32 value) throw (CannotSetValue);
	void refreshDACBox5Property(int32 index, int32 value) throw (CannotSetValue);

	/*!
	 * \brief update the progression property from driver one
	 */
	void updateProgression();


};
}
#endif // LEDPARAMCONTROLLER_H
